@@ -161,7 +161,11 @@ grn_plugin_open(grn_ctx *ctx, const char *filename)
       }
       if (!*plugin) {
         grn_hash_delete_by_id(ctx, grn_plugins, id, NULL);
-        if (!grn_dl_close(dl)) {
+        if (grn_dl_close(dl)) {
+          /* Now, __FILE__ set in plugin is invalid. */
+          ctx->errline = 0;
+          ctx->errfile = NULL;
+        } else {
           const char *label;
           label = grn_dl_close_error_label;
           SERR(label);
@@ -242,7 +246,6 @@ grn_plugins_fin(void)
 grn_rc
 grn_plugin_register_by_path(grn_ctx *ctx, const char *path)
 {
-  grn_id id;
   grn_obj *db;
   if (!ctx || !ctx->impl || !(db = ctx->impl->db)) {
     ERR(GRN_INVALID_ARGUMENT, "db not initialized");
@@ -250,6 +253,7 @@ grn_plugin_register_by_path(grn_ctx *ctx, const char *path)
   }
   GRN_API_ENTER;
   if (GRN_DB_P(db)) {
+    grn_id id = GRN_ID_NIL;
     FILE *plugin_file;
     char complemented_path[PATH_MAX], complemented_libs_path[PATH_MAX];
 
@@ -270,11 +274,11 @@ grn_plugin_register_by_path(grn_ctx *ctx, const char *path)
       } else {
         const char *base_name;
 
-        base_name = strrchr(path, '/');
+        base_name = strrchr(path, PATH_SEPARATOR[0]);
         if (base_name) {
           complemented_libs_path[0] = '\0';
           strncat(complemented_libs_path, path, base_name - path);
-          strcat(complemented_libs_path, "/.libs");
+          strcat(complemented_libs_path, PATH_SEPARATOR ".libs");
           strcat(complemented_libs_path, base_name);
           strcat(complemented_libs_path, GRN_PLUGIN_SUFFIX);
           plugin_file = fopen(complemented_libs_path, "r");
@@ -363,8 +367,30 @@ default_plugins_dir(void)
   }
   return win32_plugins_dir;
 }
+
+static void
+normalize_path_separator_in_path(char *path)
+{
+  for (; *path; path++) {
+    if (*path == '\\') {
+      *path = PATH_SEPARATOR[0];
+    }
+  }
+}
+
+static void
+normalize_path_separator_in_name(char *name)
+{
+  for (; *name; name++) {
+    if (*name == '/') {
+      *name = PATH_SEPARATOR[0];
+    }
+  }
+}
 #else /* WIN32 */
-#  define default_plugins_dir() GRN_PLUGINS_DIR;
+#  define default_plugins_dir() GRN_PLUGINS_DIR
+#  define normalize_path_separator_in_path(path)
+#  define normalize_path_separator_in_name(name)
 #endif /* WIN32 */
 
 grn_rc
@@ -373,16 +399,35 @@ grn_plugin_register(grn_ctx *ctx, const char *name)
   const char *plugins_dir;
   char dir_last_char;
   char path[PATH_MAX];
+  char normalized_name[PATH_MAX];
+  int name_length, max_name_length;
 
   plugins_dir = getenv("GRN_PLUGINS_DIR");
   if (!plugins_dir) {
     plugins_dir = default_plugins_dir();
   }
   strcpy(path, plugins_dir);
-  dir_last_char = plugins_dir[strlen(plugins_dir) - 1];
+  normalize_path_separator_in_path(path);
+
+  dir_last_char = plugins_dir[strlen(path) - 1];
   if (dir_last_char != PATH_SEPARATOR[0]) {
     strcat(path, PATH_SEPARATOR);
   }
-  strcat(path, name);
+
+  name_length = strlen(name);
+  max_name_length = PATH_MAX - strlen(path) - 1;
+  if (name_length > max_name_length) {
+    ERR(GRN_INVALID_ARGUMENT,
+        "plugin name is too long: %d (max: %d) <%s%s>",
+        name_length, max_name_length,
+        path, name);
+    return ctx->rc;
+  }
+
+  strcpy(normalized_name, name);
+  normalize_path_separator_in_name(normalized_name);
+
+  strcat(path, normalized_name);
+
   return grn_plugin_register_by_path(ctx, path);
 }
