@@ -2003,6 +2003,7 @@ proc_register(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
   if (GRN_TEXT_LEN(VAR(0))) {
     const char *name;
+    GRN_TEXT_PUTC(ctx, VAR(0), '\0');
     name = GRN_TEXT_VALUE(VAR(0));
     grn_plugin_register(ctx, name);
   } else {
@@ -2117,6 +2118,43 @@ proc_check(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
   return NULL;
 }
 
+static grn_obj *
+proc_truncate(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
+{
+  int table_name_len = GRN_TEXT_LEN(VAR(0));
+  if (table_name_len == 0) {
+      ERR(GRN_INVALID_ARGUMENT, "table name is missing");
+  } else {
+    const char *table_name = GRN_TEXT_VALUE(VAR(0));
+    grn_obj *table = grn_ctx_get(ctx, table_name, table_name_len);
+    if (!table) {
+      ERR(GRN_INVALID_ARGUMENT,
+          "no such table: <%.*s>", table_name, table_name_len);
+    } else {
+      switch (table->header.type) {
+      case GRN_TABLE_PAT_KEY :
+      case GRN_TABLE_HASH_KEY :
+      case GRN_TABLE_NO_KEY :
+        grn_table_truncate(ctx, table);
+        break;
+      default:
+        {
+          grn_obj buffer;
+          GRN_TEXT_INIT(&buffer, 0);
+          grn_inspect(ctx, &buffer, table);
+          ERR(GRN_INVALID_ARGUMENT,
+              "not a table object: %.*s",
+              GRN_TEXT_LEN(&buffer), GRN_TEXT_VALUE(&buffer));
+          GRN_OBJ_FIN(ctx, &buffer);
+        }
+        break;
+      }
+    }
+  }
+  GRN_OUTPUT_BOOL(!ctx->rc);
+  return NULL;
+}
+
 static grn_obj *
 func_rand(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
@@ -2286,7 +2324,7 @@ grn_db_init_builtin_query(grn_ctx *ctx)
   DEF_VAR(vars[13], "drilldown_offset");
   DEF_VAR(vars[14], "drilldown_limit");
   DEF_VAR(vars[15], "cache");
-  DEF_VAR(vars[16], "query_escalation_threshold");
+  DEF_VAR(vars[16], "match_escalation_threshold");
   DEF_COMMAND("define_selector", proc_define_selector, 17, vars);
   DEF_COMMAND("select", proc_select, 16, vars + 1);
 
@@ -2385,6 +2423,9 @@ grn_db_init_builtin_query(grn_ctx *ctx)
   DEF_VAR(vars[0], "obj");
   DEF_COMMAND("check", proc_check, 1, vars);
 
+  DEF_VAR(vars[0], "table");
+  DEF_COMMAND("truncate", proc_truncate, 1, vars);
+
   DEF_VAR(vars[0], "seed");
   grn_proc_create(ctx, "rand", 4, GRN_PROC_FUNCTION, func_rand, NULL, NULL, 0, vars);
 
