@@ -17,7 +17,17 @@
 #ifndef GRN_SUGGEST_UTIL_H
 #define GRN_SUGGEST_UTIL_H
 
+#include <sys/queue.h>
+#include <event.h>
+#include <stdint.h>
+
 int print_error(const char *format, ...);
 int daemonize(void);
+void parse_keyval(struct evkeyvalq *get_args,
+                  const char **query, const char **types,
+                  const char **client_id, const char **target_name,
+                  const char **learn_target_name,
+                  const char **callback,
+                  uint64_t *millisec);
 
 #endif /* GRN_SUGGEST_UTIL_H */
